/*
 *	DEFS.H
 *	Tom Kerrigan's Simple Chess Program (TSCP)
 *
 *	Copyright 1997 Tom Kerrigan
 *
 *  Traduccion al castellano por Jose Carlos Martinez Galan
 *	por sugerencia de Roger Davis
 *
 */


#define BOOL			int
#define TRUE			1
#define FALSE			0

#define MOVE_STACK		1120
#define HIST_STACK		32
#define UNDO_STACK		300

#define LIGHT			0
#define DARK			1

#define PAWN			0
#define KNIGHT			1
#define BISHOP			2
#define ROOK			3
#define QUEEN			4
#define KING			5

#define EMPTY			6

/* useful squares */
#define A1				56
#define B1				57
#define C1				58
#define D1				59
#define E1				60
#define F1				61
#define G1				62
#define H1				63
#define A8				0
#define B8				1
#define C8				2
#define D8				3
#define E8				4
#define F8				5
#define G8				6
#define H8				7

#define GetFile(x)		(x & 7)
#define GetRank(x)		(x >> 3)


/* Esta es la descripcion basica de un movimiento. promote
   es la pieza a la que promociona un peon. bits es un campo
   de bits que describe el movimiento, con los siguientes bits:

   1	captura
   2	enroque
   4	captura al paso
   8	avance de peon de 2 casillas
   16	movimiento de peon
   32	promocion

   Esta unido (union) con un integer para poder comparar
   rapida y facilmente dos movimientos */
// Los compiladores generan una comparacion mas rapida
// entre dos enteros que entre dos estructuras complejas

typedef struct {
	char from;
	char to;
	char promote;
	char bits;
} move_bytes;

typedef union {
	move_bytes b;
	long u;
} move;

/* Un elemento de la pila de jugadas. No es mas que un
   movimiento con un valor, para que pueda ser ordenado
   por las funciones de busqueda */
typedef struct {
	move m;
	long score;
} gen_t;

/* Un elemento de la tabla de historia, con la informacion
   necesaria para efectuar un retroceso */
typedef struct {
	move m;
	int capture;
	int castle;
	int ep;
	int fifty;
} hist_t;
