/*
 *	EVAL.C
 *	Tom Kerrigan's Simple Chess Program (TSCP)
 *
 *	Copyright 1997 Tom Kerrigan
 *
 *  Traduccion al castellano por Jose Carlos Martinez Galan
 *	por sugerencia de Roger Davis
 *
 */


#include <string.h>
#include "defs.h"
#include "data.h"
#include "protos.h"


#define DOUBLED_PAWN_PENALTY		10
#define ISOLATED_PAWN_PENALTY		20
#define BACKWARDS_PAWN_PENALTY		8
#define PASSED_PAWN_BONUS			20
#define ROOK_SEMI_OPEN_FILE_BONUS	10
#define ROOK_OPEN_FILE_BONUS		5
#define ROOK_ON_SEVENTH_BONUS		20


/* Los valores de las piezas */
int piece_value[6] = {
	100, 300, 300, 500, 900, 0
};

/* Los arrays "pcsq" son tablas pieza/casilla. Son valores
   que se aaden al valor material de la pieza, basandose
   en su posicion en el tablero */

int pawn_pcsq[64] = {
	  0,   0,   0,   0,   0,   0,   0,   0,
	  5,  10,  15,  20,  20,  15,  10,   5,
	  4,   8,  12,  16,  16,  12,   8,   4,
	  3,   6,   9,  12,  12,   9,   6,   3,
	  2,   4,   6,   8,   8,   6,   4,   2,
	  1,   2,   3, -10, -10,   3,   2,   1,
	  0,   0,   0, -40, -40,   0,   0,   0,
	  0,   0,   0,   0,   0,   0,   0,   0
};

int knight_pcsq[64] = {
	-10, -10, -10, -10, -10, -10, -10, -10,
	-10,   0,   0,   0,   0,   0,   0, -10,
	-10,   0,   5,   5,   5,   5,   0, -10,
	-10,   0,   5,  10,  10,   5,   0, -10,
	-10,   0,   5,  10,  10,   5,   0, -10,
	-10,   0,   5,   5,   5,   5,   0, -10,
	-10,   0,   0,   0,   0,   0,   0, -10,
	-10, -30, -10, -10, -10, -10, -30, -10
};

int bishop_pcsq[64] = {
	-10, -10, -10, -10, -10, -10, -10, -10,
	-10,   0,   0,   0,   0,   0,   0, -10,
	-10,   0,   5,   5,   5,   5,   0, -10,
	-10,   0,   5,  10,  10,   5,   0, -10,
	-10,   0,   5,  10,  10,   5,   0, -10,
	-10,   0,   5,   5,   5,   5,   0, -10,
	-10,   0,   0,   0,   0,   0,   0, -10,
	-10, -10, -20, -10, -10, -20, -10, -10
};

int king_pcsq[64] = {
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -4,  -4,  -4,  -4,  -4,  -4,  -4,  -4,
	  0,   4,   8,  -4,   0,  -4,   8,   4
};

int king_endgame_pcsq[64] = {
	  0,  10,  20,  30,  30,  20,  10,   0,
	 10,  20,  30,  40,  40,  30,  20,  10,
	 20,  30,  40,  50,  50,  40,  30,  20,
	 30,  40,  50,  60,  60,  50,  40,  30,
	 30,  40,  50,  60,  60,  50,  40,  30,
	 20,  30,  40,  50,  50,  40,  30,  20,
	 10,  20,  30,  40,  40,  30,  20,  10,
	  0,  10,  20,  30,  30,  20,  10,   0
};

/* El array flip se usa para calcular los valores de
   pieza/casilla para las negras. El valor pieza/casilla
   de un peon blanco es pawn_pcsq[sq], mientras que el de
   uno negro es pawn_pcsq[flip[sq]] */
int flip[64] = {
	 56,  57,  58,  59,  60,  61,  62,  63,
	 48,  49,  50,  51,  52,  53,  54,  55,
	 40,  41,  42,  43,  44,  45,  46,  47,
	 32,  33,  34,  35,  36,  37,  38,  39,
	 24,  25,  26,  27,  28,  29,  30,  31,
	 16,  17,  18,  19,  20,  21,  22,  23,
	  8,   9,  10,  11,  12,  13,  14,  15,
	  0,   1,   2,   3,   4,   5,   6,   7
};

/* pawn_count[x][y] es el numero de peones de color x en la columna y.
   El array tiene 2 columnas extra (0 y 9) que son siempre 0, para
   poder usar el mismo codigo de evaluacion en todos los peones sin
   preocuparnos de acceder fuera del array (cuando se comprueba el
   anterior o siguiente) */
int pawn_count[2][10];

/* pawn_rank[x][y] es la fila del peon menos avanzaso de color x
   en la columna y. Si no hay peones en esa columna, asumimos que
   hay uno fuera del tablero. Este truco facilita el funcionamiento
   de la evaluacion de peones. Este array tiene dos columnas de mas
   por las mismas razones que el anterior */
int pawn_rank[2][10];

int piece_mat[2];  /* Los valores de las piezas de cada bando */
int pawn_mat[2];  /* Los valores de los peones de cada bando */

int eval()
{
	int i;
	int f;  /* columna */
	int score[2];  /* La evaluacion de cada bando */

	/* Primera pasada: establecer pawn_count, pawn_rank,
	   piece_mat, y pawn_mat */
	for (i = 0; i < 10; ++i) {
		pawn_count[LIGHT][i] = 0;
		pawn_count[DARK][i] = 0;
		pawn_rank[LIGHT][i] = -1;
		pawn_rank[DARK][i] = 8;
	}
	piece_mat[LIGHT] = 0;
	piece_mat[DARK] = 0;
	pawn_mat[LIGHT] = 0;
	pawn_mat[DARK] = 0;
	for (i = 0; i < 64; ++i) {
		if (color[i] == EMPTY)
			continue;
		if (piece[i] == PAWN) {
			pawn_mat[color[i]] += piece_value[PAWN];
			f = GetFile(i) + 1;  /* Sumamos 1 por la columna extra en el array */
			if (color[i] == LIGHT) {
				++pawn_count[LIGHT][f];
				if (pawn_rank[LIGHT][f] < GetRank(i))
					pawn_rank[LIGHT][f] = GetRank(i);
			}
			else {
				pawn_count[DARK][f]++;
				if (pawn_rank[DARK][f] > GetRank(i))
					pawn_rank[DARK][f] = GetRank(i);
			}
		}
		else
			piece_mat[color[i]] += piece_value[piece[i]];
	}

	/* Segunda pasada: evaluar cada pieza */
	score[LIGHT] = piece_mat[LIGHT] + pawn_mat[LIGHT];
	score[DARK] = piece_mat[DARK] + pawn_mat[DARK];
	for (i = 0; i < 64; ++i) {
		if (color[i] == EMPTY)
			continue;
		if (color[i] == LIGHT) {
			switch (piece[i]) {
				case PAWN:
					score[LIGHT] += eval_light_pawn(i);
					break;
				case KNIGHT:
					score[LIGHT] += knight_pcsq[i];
					break;
				case BISHOP:
					score[LIGHT] += bishop_pcsq[i];
					break;
				case ROOK:
					if (!pawn_count[LIGHT][GetFile(i) + 1]) {
						score[LIGHT] += ROOK_SEMI_OPEN_FILE_BONUS;
						if (!pawn_count[DARK][GetFile(i) + 1])
							score[LIGHT] += ROOK_OPEN_FILE_BONUS;
					}
					if (GetRank(i) == 1)
						score[LIGHT] += ROOK_ON_SEVENTH_BONUS;
					break;
				case KING:
					if (piece_mat[DARK] <= 1200)
						score[LIGHT] += king_endgame_pcsq[i];
					else
						score[LIGHT] += eval_light_king(i);
					break;
			}
		}
		else {
			switch (piece[i]) {
				case PAWN:
					score[DARK] += eval_dark_pawn(i);
					break;
				case KNIGHT:
					score[DARK] += knight_pcsq[flip[i]];
					break;
				case BISHOP:
					score[DARK] += bishop_pcsq[flip[i]];
					break;
				case ROOK:
					if (!pawn_count[DARK][GetFile(i) + 1]) {
						score[DARK] += ROOK_SEMI_OPEN_FILE_BONUS;
						if (!pawn_count[LIGHT][GetFile(i) + 1])
							score[DARK] += ROOK_OPEN_FILE_BONUS;
					}
					if (GetRank(i) == 6)
						score[DARK] += ROOK_ON_SEVENTH_BONUS;
					break;
				case KING:
					if (piece_mat[LIGHT] <= 1200)
						score[DARK] += king_endgame_pcsq[flip[i]];
					else
						score[DARK] += eval_dark_king(i);
					break;
			}
		}
	}

	/* Ya tenemos el array score[]. Ahora hay que devolverlo teniendo
	   en cuenta a quien le toca mover */
	if (side == LIGHT)
		return score[LIGHT] - score[DARK];
	return score[DARK] - score[LIGHT];
}

int eval_light_pawn(int sq)
{
	int r;  /* Valor a devolver */
	int f;  /* Columna del peon */

	r = 0;
	f = GetFile(sq) + 1;

	r += pawn_pcsq[sq];

	/* Si hay otro peon debajo, es que esta doblado */
	if (pawn_rank[LIGHT][f] > GetRank(sq))
		r -= DOUBLED_PAWN_PENALTY;

	/* Si no hay ningun peon propio a los lados, es
	   que esta aislado */
	if ((!pawn_count[LIGHT][f - 1]) &&
			(!pawn_count[LIGHT][f + 1]))
		r -= ISOLATED_PAWN_PENALTY;

	/* Si no esta aislado, puede estar retrasado */
	else if ((pawn_rank[LIGHT][f - 1] < GetRank(sq)) &&
			(pawn_rank[LIGHT][f + 1] < GetRank(sq)))
		r -= BACKWARDS_PAWN_PENALTY;

	/* Si el peon esta pasado, se aade una bonificacion */
	if ((pawn_rank[DARK][f - 1] >= GetRank(sq)) &&
			(pawn_rank[DARK][f] >= GetRank(sq)) &&
			(pawn_rank[DARK][f + 1] >= GetRank(sq)))
		r += (8 - GetRank(sq)) * PASSED_PAWN_BONUS;

	return r;
}

int eval_dark_pawn(int sq)
{
	int r;  /* Valor a devolver */
	int f;  /* La columna del peon */

	r = 0;
	f = GetFile(sq) + 1;

	r += pawn_pcsq[flip[sq]];

	/* Si hay otro peon detras, es que esta doblado */
	if (pawn_rank[DARK][f] < GetRank(sq))
		r -= DOUBLED_PAWN_PENALTY;

	/* Si no tiene peones propios a los lados, 
	   es que esta aislado */
	if ((!pawn_count[DARK][f - 1]) &&
			(!pawn_count[DARK][f + 1]))
		r -= ISOLATED_PAWN_PENALTY;

	/* Si no esta aislado, puede estar retrasado */
	else if ((pawn_rank[DARK][f - 1] > GetRank(sq)) &&
			(pawn_rank[DARK][f + 1] > GetRank(sq)))
		r -= BACKWARDS_PAWN_PENALTY;

	/* Si el peon esta pasado, se aade una bonificacion */
	if ((pawn_rank[LIGHT][f - 1] <= GetRank(sq)) &&
			(pawn_rank[LIGHT][f] <= GetRank(sq)) &&
			(pawn_rank[LIGHT][f + 1] <= GetRank(sq)))
		r += GetRank(sq) * PASSED_PAWN_BONUS;

	return r;
}


/* Las dos funciones siguientes implementan la seguridad del rey.
   Esta es la idea:
   Primero: el valor pieza/casilla se aade a r (r es el valor
   a devolver).
   Segundo: Se evaluan los peones que hay frente al rey. Si el
     rey esta en el flaco de dama, se evaluan los peones a, b y
	 c. Si el rey esta en el flaco de rey, se evaluan los f, g y
	 h. La evaluacion de cada columna funciona asi: si no hay
	 un peon propio en la columna, se considera abierta y se
	 castiga con una penalizacion. Si hay un peon en la fila, y
	 no se ha movido, nada ocurre. Si se ha movido una casilla,
	 se penaliza. Si se ha movido mas de de una casilla, se
	 penaliza aun mas. Una vez hecho esto para las tres filas,
	 se examinan los peones enemigos. Si se estan acercando
	 demasiado al rey, se castiga con una penalizacion de
	 "avalancha de peones". Si el rey no esta en ninguno de los
	 dos flancos, es decir, si esta en las dos columnas centrales,
	 entonces se evaluan las tres columnas mas cercanas a el.
	 Una penalizacion se asigna por cada una de estas columnas que
	 este abierta.
   Finalmente, el resultado se multiplica por un factor proporcional
	 al material del que dispone el oponente. La razon de esto es que,
	 cuanto mas material tenga el rival, mas facilmente podra explotar
	 las debilidades de la poscion del rey propio */

int eval_light_king(int sq)
{
	int r;  /* Valor a devolver */
	int i;

	r = king_pcsq[sq];

	if (GetFile(sq) < 3) {
		
		/* Columna a */
		if (pawn_count[LIGHT][1]) {
			if (pawn_rank[LIGHT][1] == 6);
			else if (pawn_rank[LIGHT][1] == 5)
				r -= 2;
			else
				r -= 4;
		}
		else if (!pawn_count[DARK][1])
			r -= 8;

		/* Columna b */
		if (pawn_count[LIGHT][2]) {
			if (pawn_rank[LIGHT][2] == 6);
			else if (pawn_rank[LIGHT][2] == 5)
				r -= 2;
			else
				r -= 4;
		}
		else if (!pawn_count[DARK][2])
			r -= 8;

		/* Columna c */
		if (pawn_count[LIGHT][3]) {
			if (pawn_rank[LIGHT][3] == 6);
			else if (pawn_rank[LIGHT][3] ==5 )
				r -= 1;
			else
				r -= 3;
		}
		else if (!pawn_count[DARK][3])
			r -= 5;

		/* Avalancha de peones */
		for (i = 1; i < 4; ++i) {
			if (!pawn_count[DARK][i])
				continue;
			if (pawn_rank[DARK][i] > 2) {
				if (pawn_rank[DARK][i] > 3)
					r -= 2;
				else
					r -= 1;
			}
		}
	}
	else if (GetFile(sq) > 4) {
		
		/* Columna h */
		if (pawn_count[LIGHT][8]) {
			if (pawn_rank[LIGHT][8] == 6);
			else if (pawn_rank[LIGHT][8] == 5)
				r -= 2;
			else
				r -= 4;
		}
		else if (!pawn_count[DARK][8])
			r -= 8;

		/* Columna g */
		if (pawn_count[LIGHT][7]) {
			if (pawn_rank[LIGHT][7] == 6);
			else if (pawn_rank[LIGHT][7] == 5)
				r -= 2;
			else
				r -= 4;
		}
		else if (!pawn_count[DARK][7])
			r -= 8;

		/* Columna f */
		if (pawn_count[LIGHT][6]) {
			if (pawn_rank[LIGHT][6] == 6);
			else if (pawn_rank[LIGHT][6] == 5)
				r -= 1;
			else
				r -= 3;
		}
		else if (!pawn_count[DARK][6])
			r -= 5;

		/* Avalancha de peones */
		for (i = 6; i < 9; ++i) {
			if (!pawn_count[DARK][i])
				continue;
			if (pawn_rank[DARK][i] > 2) {
				if (pawn_rank[DARK][i] > 3)
					r -= 2;
				else
					r -= 1;
			}
		}
	}
	else {
		for (i = GetFile(sq); i <= GetFile(sq) + 2; ++i)
			if ((!pawn_count[LIGHT][i]) &&
					(!pawn_count[DARK][i]))
				r -= 2;
	}
	r *= piece_mat[DARK] / 600;
	return r;
}

int eval_dark_king(int sq)
{
	int r;  /* Valor a devolver */
	int i;

	r = king_pcsq[flip[sq]];

	if (GetFile(sq) < 3) {
		
		/* Columna a */
		if (pawn_count[DARK][1]) {
			if (pawn_rank[DARK][1] == 1);
			else if (pawn_rank[DARK][1] == 2)
				r -= 2;
			else
				r -= 4;
		}
		else if (!pawn_count[LIGHT][1])
			r -= 8;

		/* Columna b */
		if (pawn_count[DARK][2]) {
			if (pawn_rank[DARK][2] == 1);
			else if (pawn_rank[DARK][2] == 2)
				r -= 2;
			else
				r -= 4;
		}
		else if (!pawn_count[LIGHT][2])
			r -= 8;

		/* Columna c */
		if (pawn_count[DARK][3]) {
			if (pawn_rank[DARK][3] == 1);
			else if (pawn_rank[DARK][3] == 2)
				r -= 1;
			else
				r -= 3;
		}
		else if (!pawn_count[LIGHT][3])
			r -= 5;

		/* Avalancha de peones */
		for (i = 1;i<4;i++) {
			if (!pawn_count[LIGHT][i])
				continue;
			if (pawn_rank[LIGHT][i] < 5) {
				if (pawn_rank[DARK][i] < 4)
					r -= 2;
				else
					r -= 1;
			}
		}
	}
	else if (GetFile(sq) > 4) {
		
		/* Columna h */
		if (pawn_count[DARK][8]) {
			if (pawn_rank[DARK][8] == 1);
			else if (pawn_rank[DARK][8] == 2)
				r -= 2;
			else
				r -= 4;
		}
		else if (!pawn_count[LIGHT][8])
			r -= 8;

		/* Columna g */
		if (pawn_count[DARK][7]) {
			if (pawn_rank[DARK][7] == 1);
			else if (pawn_rank[DARK][7] == 2)
				r -= 2;
			else
				r -= 4;
		}
		else if (!pawn_count[LIGHT][7])
			r -= 8;

		/* Columna f */
		if (pawn_count[DARK][6]) {
			if (pawn_rank[DARK][6] == 1);
			else if (pawn_rank[DARK][6] == 2)
				r -= 1;
			else
				r -= 3;
		}
		else if (!pawn_count[LIGHT][6])
			r -= 5;

		/* Avalancha de peones */
		for (i = 6; i < 9; ++i) {
			if (!pawn_count[LIGHT][i])
				continue;
			if (pawn_rank[LIGHT][i] < 5) {
				if (pawn_rank[LIGHT][i] < 4)
					r -= 2;
				else
					r -= 1;
			}
		}
	}
	else {
		for (i = GetFile(sq); i <= GetFile(sq) + 2; ++i)
			if ((!pawn_count[LIGHT][i]) &&
					(!pawn_count[DARK][i]))
				r -= 2;
	}
	r *= piece_mat[LIGHT] / 600;
	return r;
}
