/*
 *	MAIN.C
 *	Tom Kerrigan's Simple Chess Program (TSCP)
 *
 *	Copyright 1997 Tom Kerrigan
 *
 *  Traduccion al castellano por Jose Carlos Martinez Galan
 *	por sugerencia de Roger Davis
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include "defs.h"
#include "data.h"
#include "protos.h"


/* main() es, basicamente, un bucle infinito que llama a think()
   cuando es el turno del ordenador o espera una jugada del
   usuario y la procesa */

void main()
{
	int computer_side;
	int skill_level;
	char s[256];
	int from, to, i;
	BOOL found;

	printf("\n");
	printf("Tom Kerrigan's Simple Chess Program (TSCP)\n");
	printf("version 1.42, 12/12/99\n");
	printf("Copyright 1997 Tom Kerrigan\n");
	printf("\n");
	printf("\"help\" displays a list of commands.\n");
	printf("\n");
	init();
	gen();
	computer_side = EMPTY;
	skill_level = 4; // Profundidad fija a la que se va a buscar, sin
					 // tener en cuenta el tiempo disponible
	for (;;) {
		if (side == computer_side) {  /* turno del ordenador */
			
			/* pensar sobre la jugada y realizarla */
			think(skill_level, FALSE);
			if (!pv[0][0].u) {
				printf("(no legal moves)\n");
				computer_side = EMPTY;
				continue;
			}
			printf("Computer's move: ");
			print_move(pv[0][0].b);
			printf("\n");
			makemove(pv[0][0].b);

			/* salvar una copia de la informacion, porque sera
			   reemplazada */
			undo_dat[undos] = hist_dat[0];
			++undos;

			ply = 0;
			gen();
			continue;
		}

		/* obtener entrada del usuario */
		printf("tscp> ");
		scanf("%s", s);
		if (!strcmp(s, "on")) {
			computer_side = side;
			continue;
		}
		if (!strcmp(s, "off")) {
			computer_side = EMPTY;
			continue;
		}
		if (!strcmp(s, "skill")) {
			scanf("%d", &skill_level);
			if (skill_level < 1)
				skill_level = 1;
			continue;
		}
		if (!strcmp(s, "undo")) {
			if (!undos)
				continue;
			computer_side = EMPTY;
			--undos;
			hist_dat[0] = undo_dat[undos];
			ply = 1;
			takeback();
			gen();
			continue;
		}
		if (!strcmp(s, "new")) {
			computer_side = EMPTY;
			init();
			gen();
			continue;
		}
		if (!strcmp(s, "d")) {
			print_board();
			continue;
		}
		if (!strcmp(s, "bench")) {
			computer_side = EMPTY;
			bench();
			continue;
		}
		if (!strcmp(s, "bye")) {
			printf("Share and enjoy!\n");
			exit(0);
		}
		if (!strcmp(s, "xboard")) {
			xboard();
			exit(0);
		}
		if (!strcmp(s, "help")) {
			printf("on - el programa mueve con el color que toca\n");
			printf("off - el programa deja de jugar\n");
			printf("skill n - el programa analiza n plies por jugada\n");
			printf("undo - retrocede un movimiento\n");
			printf("new - comienza una nueva partida\n");
			printf("d - muestra el tablero\n");
			printf("bench - ejecuta el test\n");
			printf("bye - sale del programa\n");
			printf("xboard - cambia a modo Xboard\n");
			printf("Introduzca las jugadas en modo coordenadas, e.g., e2e4, e7e8Q\n");
			continue;
		}

		/* El usuario introdujo una jugada? */
		from = s[0] - 'a';
		from += 8 * (8 - (s[1] - '0'));
		to = s[2] - 'a';
		to += 8 * (8 - (s[3] - '0'));

		/* Recorre las jugadas para ver si es legal */
		found = FALSE;
		for (i = 0; i < gen_end[ply]; ++i)
			if (gen_dat[i].m.b.from == from && gen_dat[i].m.b.to == to) {
				found = TRUE;

				/* Toma la pieza de promocion */
				if (gen_dat[i].m.b.bits & 32)
					switch (s[4]) {
						case 'N':
							break;
						case 'B':
							i += 1;
							break;
						case 'R':
							i += 2;
							break;
						default:
							i += 3;
							break;
					}
				break;
			}
		if (!found || !makemove(gen_dat[i].m.b))
			printf("Illegal move.\n");
		else {
			/* Salva una copia de la informacion, pues
			   sera reemplazada */
			undo_dat[undos] = hist_dat[0];
			++undos;

			ply = 0;
			gen();
		}
	}
}


/* print_move imprime una jugada */

void print_move(move_bytes m)
{
	char c;

	if (m.bits & 32) {
		switch (m.promote) {
			case KNIGHT:
				c = 'n';
				break;
			case BISHOP:
				c = 'b';
				break;
			case ROOK:
				c = 'r';
				break;
			default:
				c = 'q';
				break;
		}
		printf("%c%d%c%d%c",
				GetFile(m.from) + 'a',
				8 - GetRank(m.from),
				GetFile(m.to) + 'a',
				8 - GetRank(m.to),
				c);
	}
	else
		printf("%c%d%c%d",
				GetFile(m.from) + 'a',
				8 - GetRank(m.from),
				GetFile(m.to) + 'a',
				8 - GetRank(m.to));
}


/* print_board imprime el tablero */

void print_board()
{
	int i;
	
	printf("\n8 ");
	for (i = 0; i < 64; ++i) {
		switch (color[i]) {
			case EMPTY:
				printf(" .");
				break;
			case LIGHT:
				printf(" %c", piece_char[piece[i]]);
				break;
			case DARK:
				printf(" %c", piece_char[piece[i]] + ('a' - 'A'));
				break;
		}
		if ((i + 1) % 8 == 0 && i != 63)
			printf("\n%d ", 7 - GetRank(i));
	}
	printf("\n\n   a b c d e f g h\n\n");
}


/* xboard() substituye a main() para el modo compatible
   XBoard y WinBoard */

void xboard()
{
	int computer_side;
	int skill_level;
	char line[256], command[256];
	int from, to, i;
	BOOL found;

	signal(SIGINT, SIG_IGN);
	printf("\n");
	init();
	gen();
	computer_side = EMPTY;
	skill_level = 4;
	for (;;) {
		if (side == computer_side) {
			think(skill_level, TRUE);
			if (!pv[0][0].u) {
				computer_side = EMPTY;
				continue;
			}
			printf("move ");
			print_move(pv[0][0].b);
			printf("\n");
			fflush(stdout);
			makemove(pv[0][0].b);
			undo_dat[undos] = hist_dat[0];
			++undos;
			ply = 0;
			gen();
			continue;
		}
		fgets(line, 256, stdin);
		if (line[0] == '\n')
			continue;
		sscanf(line, "%s", command);
		if (!strcmp(command, "xboard"))
			continue;
		if (!strcmp(command, "new")) {
			init();
			gen();
			computer_side = DARK;
			continue;
		}
		if (!strcmp(command, "quit"))
			return;
		if (!strcmp(command, "force")) {
			computer_side = EMPTY;
			continue;
		}
		if (!strcmp(command, "white")) {
			side = LIGHT;
			xside = DARK;
			undos = 0;
			ply = 0;
			gen();
			computer_side = DARK;
			continue;
		}
		if (!strcmp(command, "black")) {
			side = DARK;
			xside = LIGHT;
			undos = 0;
			ply = 0;
			gen();
			computer_side = LIGHT;
			continue;
		}
		if (!strcmp(command, "sd")) {
			sscanf(line, "sd %d", &skill_level);
			continue;
		}
		if (!strcmp(command, "go")) {
			computer_side = side;
			continue;
		}
		if (!strcmp(command, "hint")) {
			think(skill_level, TRUE);
			if (!pv[0][0].u)
				continue;
			printf("Hint: ");
			print_move(pv[0][0].b);
			printf("\n");
			fflush(stdout);
			continue;
		}
		if (!strcmp(command, "undo")) {
			if (!undos)
				continue;
			--undos;
			hist_dat[0] = undo_dat[undos];
			ply = 1;
			takeback();
			gen();
			continue;
		}
		if (!strcmp(command, "remove")) {
			if (undos < 2)
				continue;
			--undos;
			hist_dat[0] = undo_dat[undos];
			ply = 1;
			takeback();
			--undos;
			hist_dat[0] = undo_dat[undos];
			ply = 1;
			takeback();
			gen();
			continue;
		}
		from = line[0] - 'a';
		from += 8 * (8 - (line[1] - '0'));
		to = line[2] - 'a';
		to += 8 * (8 - (line[3] - '0'));
		found = FALSE;
		for (i = 0; i < gen_end[ply]; ++i)
			if (gen_dat[i].m.b.from == from && gen_dat[i].m.b.to == to) {
				found = TRUE;
				if (gen_dat[i].m.b.bits & 32)
					switch (line[4]) {
						case 'n':
							break;
						case 'b':
							i += 1;
							break;
						case 'r':
							i += 2;
							break;
						default:
							i += 3;
							break;
					}
				break;
			}
		if (!found || !makemove(gen_dat[i].m.b)) {
			printf("Error (unknown command): %s\n", command);
			fflush(stdout);
		}
		else {
			undo_dat[undos] = hist_dat[0];
			++undos;
			ply = 0;
			gen();
		}
	}
}


/* bench: Es un pequeo codigo que calcula cuantos nodos por segundo
   calcula TSCP. Establece la posicion en la jugada 17 de la partida
   Bobby Fischer vs. J. Sherwin, New Jersey State Open Championship,
   9/2/1957. Despues busca 5 plies 3 veces. Calcula los nodos por
   segundo de la mejor vez */

int bench_color[64] = {
	6, 1, 1, 6, 6, 1, 1, 6,
	1, 6, 6, 6, 6, 1, 1, 1,
	6, 1, 6, 1, 1, 6, 1, 6,
	6, 6, 6, 1, 6, 6, 0, 6,
	6, 6, 1, 0, 6, 6, 6, 6,
	6, 6, 0, 6, 6, 6, 0, 6,
	0, 0, 0, 6, 6, 0, 0, 0,
	0, 6, 0, 6, 0, 6, 0, 6
};

int bench_piece[64] = {
	6, 3, 2, 6, 6, 3, 5, 6,
	0, 6, 6, 6, 6, 0, 0, 0,
	6, 0, 6, 4, 0, 6, 1, 6,
	6, 6, 6, 1, 6, 6, 1, 6,
	6, 6, 0, 0, 6, 6, 6, 6,
	6, 6, 0, 6, 6, 6, 0, 6,
	0, 0, 4, 6, 6, 0, 2, 0,
	3, 6, 2, 6, 3, 6, 5, 6
};

BOOL ftime_works;

void bench()
{
	int i;
	long start, t[3];
	double nps;

	for (i = 0; i < 64; ++i) {
		color[i] = bench_color[i];
		piece[i] = bench_piece[i];
	}
	side = LIGHT;
	xside = DARK;
	castle = 0;
	ep = -1;
	fifty = 0;
	ply = 0;
	undos = 0;

	ftime_works = FALSE;
	for (i = 0; i < 3; ++i) {
		start = get_ms();
		think(5, FALSE);
		t[i] = get_ms() - start;
		printf("Time: %ld ms\n", t[i]);
	}
	if (!ftime_works) {
		printf("\n");
		printf("ftime() isn't returning milliseconds.\n");
		printf("Please change get_ms() in main.c to be more accurate.\n");
		printf("\n");
		return;
	}
	if (t[1] < t[0])
		t[0] = t[1];
	if (t[2] < t[0])
		t[0] = t[2];
	printf("\n");
	printf("Nodes: %ld\n", nodes);
	printf("Best time: %ld ms\n", t[0]);
	if (t[0] == 0) {
		printf("(invalid)\n");
		return;
	}
	nps = (double)nodes;
	nps /= (double)t[0];
	nps *= 1000.0;
	printf("Nodes per second: %d\n", (int)nps);
}


/* get_ms() devuelve los millisegundos transcurridos desde
   medianoche del 1 de enero de 1970 */

#include <sys/timeb.h>
long get_ms()
{
	struct timeb timebuffer;
	ftime(&timebuffer);
	if (timebuffer.millitm)
		ftime_works = TRUE;
	return (timebuffer.time * 1000) + timebuffer.millitm;
}
